import os
import glob
import unittest
from distutils.core import setup, Extension

TEST_EXTENSIONS = [
    Extension('_test_extension',
              [os.path.join('tests', '_test_extension.c')],
              include_dirs=[os.path.curdir]),
]

if os.environ.get('GREENLET_TEST_CPP', 'yes').lower() not in ('0', 'no', 'false'):
    TEST_EXTENSIONS_CPP = [
        Extension('_test_extension_cpp',
                  [os.path.join('tests', '_test_extension_cpp.cpp')],
                  language="c++",
                  include_dirs=[os.path.curdir]),
    ]
else:
    TEST_EXTENSIONS_CPP = []


def test_collector(build_base=None):
    """Collect all tests under the tests directory and return a
    unittest.TestSuite
    """
    build_test_extensions(build_base)
    tests_dir = os.path.realpath(os.path.dirname(__file__))
    test_module_list = [
        'tests.%s' % os.path.splitext(os.path.basename(t))[0]
        for t in glob.glob(os.path.join(tests_dir, 'test_*.py'))]
    if not TEST_EXTENSIONS_CPP:
        test_module_list.remove('tests.test_cpp')
    return unittest.TestLoader().loadTestsFromNames(test_module_list)


def build_test_extensions(build_base=None):
    """Because distutils sucks, it just copies the entire contents of the build
    results dir (e.g. build/lib.linux-i686-2.6) during installation. That means
    that we can't put any files there that we don't want to distribute.

    To deal with it, this code will compile test extensions in a separate
    directory, prepending it to sys.path afterwards. This way testing with
    multiple Python release and pydebug versions works and test extensions
    are not distributed.
    """
    if build_base is None:
        build_base = os.path.join('build', 'tests')
    from my_build_ext import build_ext
    setup(
        options={
            'build': {'build_base': build_base},
        },
        cmdclass=dict(build_ext=build_ext),
        script_args=['-q', 'build_ext', '-q'],
        ext_modules=TEST_EXTENSIONS + TEST_EXTENSIONS_CPP)



import unittest
import sys
import gc

import time
import weakref
import greenlet
import threading


class ArgRefcountTests(unittest.TestCase):
    def test_arg_refs(self):
        args = ('a', 'b', 'c')
        refcount_before = sys.getrefcount(args)
        g = greenlet.greenlet(
            lambda *args: greenlet.getcurrent().parent.switch(*args))
        for i in range(100):
            g.switch(*args)
        self.assertEqual(sys.getrefcount(args), refcount_before)

    def test_kwarg_refs(self):
        kwargs = {}
        g = greenlet.greenlet(
            lambda **kwargs: greenlet.getcurrent().parent.switch(**kwargs))
        for i in range(100):
            g.switch(**kwargs)
        self.assertEqual(sys.getrefcount(kwargs), 2)

    if greenlet.GREENLET_USE_GC:
        # These only work with greenlet gc support

        def recycle_threads(self):
            # By introducing a thread that does sleep we allow other threads,
            # that have triggered their __block condition, but did not have a
            # chance to deallocate their thread state yet, to finally do so.
            # The way it works is by requring a GIL switch (different thread),
            # which does a GIL release (sleep), which might do a GIL switch
            # to finished threads and allow them to clean up.
            def worker():
                time.sleep(0.001)
            t = threading.Thread(target=worker)
            t.start()
            time.sleep(0.001)
            t.join()

        def test_threaded_leak(self):
            gg = []
            def worker():
                # only main greenlet present
                gg.append(weakref.ref(greenlet.getcurrent()))
            for i in range(2):
                t = threading.Thread(target=worker)
                t.start()
                t.join()
                del t
            greenlet.getcurrent() # update ts_current
            self.recycle_threads()
            greenlet.getcurrent() # update ts_current
            gc.collect()
            greenlet.getcurrent() # update ts_current
            for g in gg:
                self.assertTrue(g() is None)

        def test_threaded_adv_leak(self):
            gg = []
            def worker():
                # main and additional *finished* greenlets
                ll = greenlet.getcurrent().ll = []
                def additional():
                    ll.append(greenlet.getcurrent())
                for i in range(2):
                    greenlet.greenlet(additional).switch()
                gg.append(weakref.ref(greenlet.getcurrent()))
            for i in range(2):
                t = threading.Thread(target=worker)
                t.start()
                t.join()
                del t
            greenlet.getcurrent() # update ts_current
            self.recycle_threads()
            greenlet.getcurrent() # update ts_current
            gc.collect()
            greenlet.getcurrent() # update ts_current
            for g in gg:
                self.assertTrue(g() is None)



import gc
import greenlet
import weakref
import unittest


class WeakRefTests(unittest.TestCase):
    def test_dead_weakref(self):
        def _dead_greenlet():
            g = greenlet.greenlet(lambda: None)
            g.switch()
            return g
        o = weakref.ref(_dead_greenlet())
        gc.collect()
        self.assertEqual(o(), None)

    def test_inactive_weakref(self):
        o = weakref.ref(greenlet.greenlet())
        gc.collect()
        self.assertEqual(o(), None)

    def test_dealloc_weakref(self):
        seen = []
        def worker():
            try:
                greenlet.getcurrent().parent.switch()
            finally:
                seen.append(g())
        g = greenlet.greenlet(worker)
        g.switch()
        g2 = greenlet.greenlet(lambda: None, g)
        g = weakref.ref(g2)
        g2 = None
        self.assertEqual(seen, [None])



#! /usr/bin/env python

import sys, os, getopt, struct, unittest
from distutils.spawn import spawn

build = True
verbosity = 2
build_base = None
here = os.path.dirname(os.path.abspath(__file__))
os.chdir(here)


def bits():
    """determine if running on a 32 bit or 64 bit platform
    """
    return struct.calcsize("P") * 8

# -- parse options
try:
    opts, args = getopt.getopt(sys.argv[1:], "nqb:")
    if args:
        raise getopt.GetoptError("too many arguments")
except getopt.GetoptError:
    sys.exit("run-tests.py: error: %s" % sys.exc_info()[1])

for o, a in opts:
    if o == "-q":
        verbosity = 0
    elif o == "-n":
        build = False
    elif o == "-b":
        build_base = a

# -- build greenlet
if build:
    if verbosity == 0:
        cmd = [sys.executable, "setup.py", "-q", "build_ext", "-q", "-i"]
    else:
        cmd = [sys.executable, "setup.py", "build_ext", "-i"]

    spawn(cmd, search_path=0)


# -- find greenlet but skip the one in "."
if not build:
    oldpath = sys.path[:]
    sys.path.remove(here)

import greenlet

if not build:
    sys.path[:] = oldpath

sys.stdout.write("python %s (%s bit) using greenlet %s from %s\n" %
                 (sys.version.split()[0], bits(), greenlet.__version__, greenlet.__file__))
sys.stdout.flush()


# -- run tests
from tests import test_collector
result = unittest.TextTestRunner(verbosity=verbosity).run(test_collector(build_base))
if result.failures or result.errors:
    sys.exit(1)



# configuration file for py.test

import sys, os
from distutils.spawn import spawn


def pytest_configure(config):
    os.chdir(os.path.dirname(__file__))
    cmd = [sys.executable, "setup.py", "-q", "build_ext", "-q", "-i"]
    spawn(cmd, search_path=0)

    from tests import build_test_extensions
    build_test_extensions()


def pytest_report_header(config):
    import greenlet
    return "greenlet %s from %s" % (greenlet.__version__, greenlet.__file__)



import unittest

import greenlet
import _test_extension_cpp


class CPPTests(unittest.TestCase):
    def test_exception_switch(self):
        greenlets = []
        for i in range(4):
            g = greenlet.greenlet(_test_extension_cpp.test_exception_switch)
            g.switch(i)
            greenlets.append(g)
        for i, g in enumerate(greenlets):
            self.assertEqual(g.switch(), i)



#!/usr/bin/env python

"""Create a chain of coroutines and pass a value from one end to the other,
where each coroutine will increment the value before passing it along.
"""

import optparse
import time

import greenlet


def link(next_greenlet):
    value = greenlet.getcurrent().parent.switch()
    next_greenlet.switch(value + 1)


def chain(n):
    start_node = greenlet.getcurrent()
    for i in xrange(n):
        g = greenlet.greenlet(link)
        g.switch(start_node)
        start_node = g
    return start_node.switch(0)

if __name__ == '__main__':
    p = optparse.OptionParser(
        usage='%prog [-n NUM_COROUTINES]', description=__doc__)
    p.add_option(
        '-n', type='int', dest='num_greenlets', default=100000,
        help='The number of greenlets in the chain.')
    options, args = p.parse_args()

    if len(args) != 0:
        p.error('unexpected arguments: %s' % ', '.join(args))

    start_time = time.clock()
    print 'Result:', chain(options.num_greenlets)
    print time.clock() - start_time, 'seconds'



# this code has been taken from gevent's setup.py file. it provides a
# build_ext command that puts .so/.pyd files in-place (like "setup.py
# build_ext -i"). it uses symlinks if possible and will rebuild when
# changing the python version (unlike "setup.py build_ext -i")

import sys, os, shutil

from distutils.command.build_ext import build_ext as _build_ext


def symlink_or_copy(src, dst):
    if hasattr(os, 'symlink'):
        try:
            os.symlink(src, dst)
            return
        except OSError:  # symbolic link privilege not held??
            pass
        except NotImplementedError:  # running on XP/'CreateSymbolicLinkW not found'
            pass

    shutil.copyfile(src, dst)


class build_ext(_build_ext):
    """Command for building extensions

    Prepends library directory to sys.path on normal builds (for tests).
    Otherwise it forces a non-inplace build and symlinks libraries instead.
    """

    def initialize_options(self):
        self.my_inplace = None
        _build_ext.initialize_options(self)

    def finalize_options(self):
        if self.my_inplace is None:
            self.my_inplace = self.inplace
            self.inplace = 0
        _build_ext.finalize_options(self)

    def build_extension(self, ext):
        _build_ext.build_extension(self, ext)
        if not self.my_inplace:
            build_lib = os.path.abspath(self.build_lib)
            if build_lib not in sys.path:
                sys.path.insert(0, build_lib)
            return
        filename = self.get_ext_filename(ext.name)
        build_path = os.path.abspath(os.path.join(self.build_lib, filename))
        src_path = os.path.abspath(filename)
        if build_path != src_path:
            try:
                os.unlink(src_path)
            except OSError:
                pass

            if self.verbose:
                sys.stderr.write('Linking %s to %s\n' % (build_path, src_path))

            symlink_or_copy(build_path, src_path)



#! /usr/bin/env python

import sys, os, glob, platform, tempfile, shutil

# workaround segfaults on openbsd and RHEL 3 / CentOS 3 . see
# https://bitbucket.org/ambroff/greenlet/issue/11/segfault-on-openbsd-i386
# https://github.com/python-greenlet/greenlet/issues/4
# https://github.com/python-greenlet/greenlet/issues/94
if ((sys.platform == "openbsd4" and os.uname()[-1] == "i386")
    or ("-with-redhat-3." in platform.platform() and platform.machine() == 'i686')
    or (sys.platform == "sunos5" and os.uname()[-1] == "sun4v")
    or ("SunOS" in platform.platform() and platform.machine() == "sun4v")
    or (sys.platform == "linux" and platform.machine() == "ppc")):
    os.environ["CFLAGS"] = ("%s %s" % (os.environ.get("CFLAGS", ""), "-Os")).lstrip()

try:
    if not (sys.modules.get("setuptools")
            or "develop" in sys.argv
            or "upload" in sys.argv
            or "bdist_egg" in sys.argv
            or "bdist_wheel" in sys.argv
            or "test" in sys.argv):
        raise ImportError()
    from setuptools import setup, Extension
    setuptools_args = dict(test_suite='tests.test_collector', zip_safe=False)
except ImportError:
    from distutils.core import setup, Extension
    setuptools_args = dict()

def readfile(filename):
    f = open(filename)
    try:
        return f.read()
    finally:
        f.close()

def _find_platform_headers():
    return glob.glob("platform/switch_*.h")

if hasattr(sys, "pypy_version_info"):
    ext_modules = []
    headers = []
else:
    headers = ['greenlet.h']

    if sys.platform == 'win32' and '64 bit' in sys.version:
        # this works when building with msvc, not with 64 bit gcc
        # switch_x64_masm.obj can be created with setup_switch_x64_masm.cmd
        extra_objects = ['platform/switch_x64_masm.obj']
    else:
        extra_objects = []

    if sys.platform == 'win32' and os.environ.get('GREENLET_STATIC_RUNTIME') in ('1', 'yes'):
        extra_compile_args = ['/MT']
    elif hasattr(os, 'uname') and os.uname()[4] in ['ppc64el', 'ppc64le']:
        extra_compile_args = ['-fno-tree-dominator-opts']
    else:
        extra_compile_args = []

    ext_modules = [Extension(
        name='greenlet',
        sources=['greenlet.c'],
        extra_objects=extra_objects,
        extra_compile_args=extra_compile_args,
        depends=['greenlet.h', 'slp_platformselect.h'] + _find_platform_headers())]

from distutils.core import Command
from my_build_ext import build_ext


setup(
    name="greenlet",
    version='0.4.16',
    description='Lightweight in-process concurrent programming',
    long_description=readfile("README.rst"),
    maintainer="Alexey Borzenkov",
    maintainer_email="snaury@gmail.com",
    url="https://github.com/python-greenlet/greenlet",
    license="MIT License",
    platforms=['any'],
    headers=headers,
    ext_modules=ext_modules,
    cmdclass=dict(build_ext=build_ext),
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: C',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.4',
        'Programming Language :: Python :: 2.5',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.0',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Python Modules'],
    **setuptools_args)



#! /usr/bin/env python

import sys, os, re, unittest, greenlet

def readclose(f):
    try:
        return f.read()
    finally:
        f.close()

def readfile(filename):
    return readclose(open(filename))

class VersionTests(unittest.TestCase):
    def test_version(self):
        upfile = lambda p: os.path.join(os.path.dirname(__file__), "..", p)
        hversion, = re.findall('GREENLET_VERSION "(.*)"', readfile(upfile("greenlet.h")))
        sversion = readclose(os.popen("%s %s --version" % (sys.executable, upfile("setup.py")))).strip()
        self.assertFalse(sversion != hversion)
        self.assertFalse(sversion != greenlet.__version__)
